#include <iostream>
#include <string>
#include <algorithm>
#include <cstdlib>
#include <socklibpp/socklistener.h>

using namespace std;
using namespace socklibpp;

listener<> server;
int global_counter = 1;

#define USE_FOREACH

bool do_stuffs(sock sb, int i)
{
        if (i == 0) {
                addr_in a;
                sock newconn = sb.accept(a);
                if (!sb.error) {        // No error
                        cout << "Received connection " << global_counter <<
                                '(' << newconn.fd << ')' << " from " << a << endl;
                        server.add(newconn, global_counter++);
                }
        } else {
                char buffer[256];
                int r = sb.recv(buffer, sizeof buffer);
                if (r<1) {
                        cout << "Lost connection " << i << '(' << sb.fd << ')' << endl;
                        sb.close();
                        #ifndef USE_FOREACH
                        // If we aren't using for_each_read, we need to remove it explicitly.
                        server.remove(sb);
                        #endif
                        return true;
                }

                cout << "Receiving data from connection " << i << '(' << sb.fd << ')' << ":\n" <<
                        string(buffer, r) << endl;
        }
        return false;
}

void do_stuffs_helper(const pair<sock,int>& data)
{
        do_stuffs(data.first, data.second);
}


int main(int argc, char **argv)
{
        uint16_t port = 3747;
        
        if (argc>1)
                port = strtoul(argv[1], NULL, 10);
        
        addr_in local(addr_in::addr_any, port);
        sock passive(sock::pf_inet, sock::sock_stream);
        if (!passive.bind(local) || !passive.listen())
                return 1;

        server.add(passive);
        
        #ifndef USE_FOREACH
        list< pair<sock, int> > to_read;
        #endif

        cout << "We are going to test the server using "
        #ifdef USE_FOREACH
                "server::for_each_read"
        #else
                "server::readable"
        #endif
               " on port " << port << endl;

        do {
                cout << "Listening to " << server.size() << " sockets.\n"
                        "Press enter to test for readability (Ctrl+C to exit)" << endl;
                cin.ignore();

                #ifndef USE_FOREACH
                to_read.clear();
                server.readable(to_read);
                for_each(to_read.begin(), to_read.end(), do_stuffs_helper);
                #else
                server.for_each_read(do_stuffs);
                #endif

        } while (cin.good());
}
